﻿using UnityEngine;
using System.Collections;

public class Blinky : Ghost {

	//see update() for description of "Cruise Elroy"
	protected enum BlinkyModes {NORMAL, CruiseElroy1, CruiseElroy2};

	protected BlinkyModes blinky_mode = BlinkyModes.NORMAL;

	float[] elroy_speeds = new float[21]{
		0.8f,0.9f,0.9f,0.9f,1.0f,1.0f,1.0f,
		1.0f,1.0f,1.0f,1.0f,1.0f,1.0f,1.0f,
		1.0f,1.0f,1.0f,1.0f,1.0f,1.0f,1.0f
	};
	int[] elroy_dots = new int[21]{20,30,40,40,40,50,50,50,60,60,60,80,80,80,100,100,100,100,120,120,120};

	public Blinky(bool menu_static) {
		//Blinky's specific starting data
		primaryColor = new Color(253/255f,36/255f,13/255f);
		start_mode = GhostMode.SCATTER;
		start_dir = _dir = move_dir = Dir.LEFT;
		startingPos.x = 14f;
		startingPos.y = -11.5f;
		homePos.x = 26;
		homePos.y = -4;
		name = "Blinky";

		if(menu_static)
			menuStatic();
		else
			init ();
	}


	public override void update (float deltaTime)
	{
		//Blinky has a special mode called "Cruise Elroy". After a certain number of dots have been eaten (level dependent and counting the number of dots remaining) Blinky will get a bit faster.
		//in later level, Blinky can actually move faster than Pac-man
		if(blinky_mode < BlinkyModes.CruiseElroy2){
			//there are two speeds to Cruise Elroy (and therefore twi different trigger conditions. In the original the speed of the siren is linked to Cruis Elroy
			if(blinky_mode == BlinkyModes.NORMAL){
				if(GameManager.Instance.dots < elroy_dots[GameManager.Instance.levelIndex]){
					//switch it up to Curise Elroy!
					blinky_mode = BlinkyModes.CruiseElroy1;
					speed = elroy_speeds[GameManager.Instance.levelIndex];
				}
			} else {
				if(GameManager.Instance.dots < elroy_dots[GameManager.Instance.levelIndex]/2){ //half as many dots again
					//switch it up another notch of Curise Elroy!!! BLINKY'S GONNA CATCH YOU PAC-MAN!
					blinky_mode = BlinkyModes.CruiseElroy2;
					speed = elroy_speeds[GameManager.Instance.levelIndex] + 0.05f; //an extra 5% faster
				}
			}
		}
		base.update (deltaTime);
	}

	protected override void findTarget ()
	{
		//Blinky is the most direct of the ghosts. When in chase mode Blinky's target is simply the tile Pac-Man is currently on
		if(_mode != GhostMode.EYES1 && _mode == GhostMode.CHASE){
			target = GameManager.Instance.pacman.pos;
		} else {
			base.findTarget ();
		}
	}
}
