﻿using UnityEngine;
using System.Collections;

//directions
public enum Dir { LEFT, RIGHT, UP, DOWN, NONE };

//base class for Ghosts and Pacman
//Mainly deals with positioning, and animation frame calculating
//all pretty simple stuff
public class Entity {

	public string name = "defaultEntity";
	public GameObject mesh;
	public Vector2 startingPos = new Vector3(0,0);

	protected int anim_frame = 0;
	protected float last_anim_tick = 0f;
	protected float frame_rate = 1/8f;
	protected int max_frames = 2;
	
	protected float maxSpeed = 8;
	protected float speed = 0.75f;
	protected float last_speed;

	protected float x;
	protected float y;
	protected int tile_x; //current tile
	protected int tile_y;
	protected int last_x; //tile last frame
	protected int last_y;
	protected int tile_id;
	protected Vector2 next_tile;
	
	protected Dir _dir = Dir.LEFT;
	protected Dir next_move_dir = Dir.LEFT;
	protected Dir move_dir = Dir.LEFT;
	protected Dir start_dir = Dir.LEFT;
	
	static protected Color32[] _blankStamp = null;
	public const int SIZE = 16;
	protected bool animating = true;
	protected bool _halt = false;

	public Entity(){

	}

	protected virtual void init(){
		x = startingPos.x;
		y = -startingPos.y;

		tile_x = (int)x;
		tile_y = (int)y;
		
		last_anim_tick = Time.time;
		mesh.renderer.material.shader = Shader.Find("Custom/simpleFrag");
		mesh.renderer.material.SetColor("_Color", Color.white);
		mesh.SetActive(false);
	}

	public virtual void firsPositions(){
		x = startingPos.x;
		y = -startingPos.y;
		
		tile_x = (int)x;
		tile_y = (int)y;

		//Debug.Log ("FIRST POSITIONS");
	}

	public void setPosition(float _x, float _y){
		x = _x;
		y = -_y;
		
		tile_x = (int)x;
		tile_y = (int)y;
		
		last_x = tile_x;
		last_y = tile_y;

		Vector3 p = mesh.transform.position;
		p.x = x;
		p.y = -y;
		mesh.transform.position = p;
	}

	public Dir moveDir {
		get { return move_dir; }
		protected set { }
	}

	public Vector2 pos {
		get {return new Vector2(tile_x, tile_y); }
	}

	public Vector2 rawPos {
		get {return new Vector2(x, y); }
	}

	public virtual void update(float deltaTime){
		calcAnimFrame();

		if(x < -1.5f){
			x = 29.4f;
			next_tile.x = 27f;
		}  else if (x > 29.5f){
			x = -1.4f;
			next_tile.x = 1f;
		}
		
		last_x = tile_x;
		last_y = tile_y;
		
		if(!(tile_x != last_x && tile_y != last_y)){
			tile_id = GameManager.Instance.map.getTileId(tile_x, tile_y);
		}
		tile_x = (int)x;
		tile_y = (int)y;
		
		Vector3 p = mesh.transform.position;
		p.x = x;
		p.y = -y;
		mesh.transform.position = p;
	}

	public virtual void halt(){
		_halt = true;
	}

	protected Color32[] blankStamp {
		get { 
			if(_blankStamp == null){
				Color32 alpha = new Color32(0,0,0,0);
				_blankStamp = new Color32[SIZE*SIZE];
				for(int i = 0; i < SIZE*SIZE; i++){
					_blankStamp[i] = alpha;
				}
			}
			return _blankStamp;
		}
	}

	protected virtual void updateDirection(){
		updateAnimFrame();
	}

	protected virtual void updateAnimFrame(){}

	protected virtual void calcAnimFrame(){
		if(animating){
			if(Time.time - last_anim_tick > frame_rate){
				anim_frame++;
				if(anim_frame >= max_frames){
					endOfAnim();
				}
				updateAnimFrame();
				last_anim_tick = Time.time;
			}
		}
	}

	protected virtual void endOfAnim(){
		anim_frame = 0;
	}

	public bool enabled {
		get { return mesh != null ? mesh.activeSelf : false; }
		set { if(mesh != null) mesh.SetActive(value); }
	}

	public int tileX { get { return tile_x; }}
	public int tileY { get { return tile_y; }}
}
