﻿using UnityEngine;
using System.Collections.Generic;

public class Pacman : Entity {

	//pacman specific mmdes
	public enum Modes {STOP, MOVE, KILL, DYING, DEAD};

	//this is a simple three frames of pacmans movement
	protected int[,] pixelData = new int[SIZE,SIZE]{
		{ 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
		{ 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0},
		{ 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 3, 3, 0, 0, 0, 0},
		{ 0, 0, 0, 1, 1, 1, 1, 1, 1, 3, 3, 3, 3, 0, 0, 0},
		{ 0, 0, 1, 1, 1, 1, 1, 1, 3, 3, 3, 3, 3, 3, 0, 0},
		{ 0, 0, 1, 1, 1, 1, 1, 3, 3, 3, 3, 3, 2, 2, 0, 0},
		{ 0, 1, 1, 1, 1, 1, 3, 3, 3, 3, 2, 2, 2, 2, 2, 0},
		{ 0, 1, 1, 1, 1, 3, 3, 3, 2, 2, 2, 2, 2, 2, 2, 0},
		{ 0, 1, 1, 1, 1, 3, 3, 3, 2, 2, 2, 2, 2, 2, 2, 0},
		{ 0, 1, 1, 1, 1, 1, 3, 3, 3, 3, 2, 2, 2, 2, 2, 0},
		{ 0, 0, 1, 1, 1, 1, 1, 3, 3, 3, 3, 3, 2, 2, 0, 0},
		{ 0, 0, 1, 1, 1, 1, 1, 1, 3, 3, 3, 3, 3, 3, 0, 0},
		{ 0, 0, 0, 1, 1, 1, 1, 1, 1, 3, 3, 3, 3, 0, 0, 0},
		{ 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 3, 3, 0, 0, 0, 0},
		{ 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0},
		{ 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}
	};

	//while this on the otherhand is truely insane
	// this is the death animation, 11 frames in all. Eleven. ELEVEN!
	// there are 40 different pixel states here. It's a bit of a pain to debug. But hey... it works!
	protected int[,] pixelDeathData = new int[SIZE,SIZE]{
		{ 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
		{ 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
		{ 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
		{ 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
		{ 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
		{ 0, 0, 0, 0, 0,35, 0, 0, 0, 0,35, 0, 0, 0, 0, 0},
		{ 0,29,29, 0, 0, 0,35, 0, 0,35, 0, 0,35,29,29, 0},
		{ 0,22,29,37, 0, 0, 0, 0, 0, 0, 0,35,29,29,22, 0},
		{28,22,22,22,36, 0, 0, 7, 7, 0, 0,29,22,22,22,28},
		{27,19,19,22,22,29,13,12,12,13,29,22,33,34,19,27},
		{26,18,39,39,17,16,15,14,14,15,16,17,18,18,18,26},
		{11,25,10, 5, 5, 4, 3, 2, 2, 3, 4,32, 5,10,25,11},
		{21,24, 5, 5,31, 4, 3, 2, 2, 3, 4, 4,32, 5,24,21},
		{ 0,40,24,31, 4, 3,30, 8, 8,30, 3, 4, 4,24,40, 0},
		{ 0, 0, 0,20,23,31, 9, 1, 1, 9,31,23,20, 6, 0, 0},
		{ 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}
	};

	protected Color yellow = new Color(255/255f, 255/255f, 0/255f);

	//
	protected Dictionary<Modes, Texture2D[]> frames = new Dictionary<Modes, Texture2D[]>();
	protected Modes mode = Modes.MOVE;
	protected Modes anim_mode = Modes.MOVE;
	protected Dictionary<Dir, Vector3> euler_rotations;
	protected Dir aux_move_dir = Dir.NONE;
	

	//ghosts aren't the only ones to change their speed in FRIGHTENED mode
	protected float[] fright_speeds = new float[21]{
		0.9f,0.95f,0.95f,0.95f,1.0f,1.0f,1.0f,
		1.0f,1.0f,1.0f,1.0f,1.0f,1.0f,1.0f,
		1.0f,1.0f,1.0f,1.0f,1.0f,1.0f,1.0f
	};

	protected float[] level_speeds = new float[21]{
		0.8f,0.9f,0.9f,0.9f,1.0f,1.0f,1.0f,
		1.0f,1.0f,1.0f,1.0f,1.0f,1.0f,1.0f,
		1.0f,1.0f,1.0f,1.0f,1.0f,1.0f,0.9f
	};

	public Pacman() : base(){

		//going to be lazy here and just rotate the transform. eww... I got unity in my code!
		euler_rotations = new Dictionary<Dir, Vector3>();
		euler_rotations.Add(Dir.RIGHT, new Vector3());
		euler_rotations.Add(Dir.LEFT, new Vector3(0,0,180));
		euler_rotations.Add(Dir.UP, new Vector3(0,0,90));
		euler_rotations.Add(Dir.DOWN, new Vector3(0,0,270));

		max_frames = 12;
		startingPos.x = 14f;
		startingPos.y = -23.5f;
		frame_rate = 1/24f;

		next_tile = new Vector2(13f,23f);
		init();
	}

	protected override void init ()
	{
		//same deal as the ghosts. make the GameObject, paint the frames and store em.
		mesh = GameObject.CreatePrimitive(PrimitiveType.Quad);
		mesh.name = "Pac-Man";

		mesh.transform.localScale = new Vector3(2,2,1);
		mesh.transform.position = new Vector3(startingPos.x,startingPos.y,-5f); 
		move_dir = Dir.LEFT;

		Texture2D[] move = new Texture2D[3];
		move[0] = Painter(pixelData, new int[]{1,2,3});
		move[1] = Painter(pixelData, new int[]{1,3});
		move[2] = Painter(pixelData, new int[]{1});
		frames.Add(Modes.MOVE,move);

		Texture2D f1 = Painter(pixelDeathData, new int[]{1,2,3,4,5,8,9,10,15,16,17,18,19,22,29,30,31,32,33,34,36,37,38,39});
		Texture2D fl = Painter(pixelDeathData, new int[]{30,31,32,33,34,35,36,37,38,39});
		Texture2D[] death = new Texture2D[]{
			f1,f1,f1,
			Painter(pixelDeathData, new int[]{2,3,4,5,8,9,10,15,16,17,18,19,22,23,24,25,26,27,28,30,31,32,33,34,38,39}),
			Painter(pixelDeathData, new int[]{2,3,4,5,8,9,10,11,16,17,18,19,23,24,25,26,27,30,31,32,34,38,39}),
			Painter(pixelDeathData, new int[]{2,3,4,5,8,9,10,11,18,21,23,24,25,26,30,31,32,39,40}),
			Painter(pixelDeathData, new int[]{2,3,4,5,8,9,10,11,14,15,16,17,21,23,24,25,30,31,32,40}),
			Painter(pixelDeathData, new int[]{2,3,4,5,10,12,13,14,15,16,17,21,23,24,25,30,31,32,38,40}),
			Painter(pixelDeathData, new int[]{2,3,4,5,6,7,8,12,13,14,15,16,17,23,24,30,31,32,40}),
			Painter(pixelDeathData, new int[]{2,3,4,7,8,9,12,13,14,15,16,23,30,31}),
			Painter(pixelDeathData, new int[]{2,3,8,9,12,14,15,30}),
			Painter(pixelDeathData, new int[]{1,2,8,12,14}),
			fl,fl,fl,
			Painter(pixelDeathData, new int[]{})
		};
		frames.Add(Modes.DYING,death);
		max_frames = death.Length;

		updateDirection();
		base.init ();
		
		last_x = tile_x;
		last_y = tile_y;
	}

	public void startLevel(){
		firsPositions();
	}

	//time for the star of the show to take their spot on stage
	public override void firsPositions ()
	{
		base.firsPositions ();
		enabled = true;
		speed = level_speeds[GameManager.Instance.levelIndex];
		mesh.transform.position = new Vector3(startingPos.x, startingPos.y, -4);
		move_dir = Dir.LEFT;
		setMode(Modes.MOVE);
		updateDirection();
		updateAnimFrame();

		
		next_tile = new Vector2((int)startingPos.x, (int)startingPos.y);
	}

	protected Texture2D Painter(int[,] data, int[] body){
		Texture2D tex = new Texture2D(data.GetLength(1),data.GetLength(0));
		tex.filterMode = FilterMode.Point;
		tex.SetPixels32(blankStamp);
		tex.Apply();
		
		for(int y = 0; y < data.GetLength(1); y++){
			for(int x = 0; x < data.GetLength(0); x++){
				int c = data[y,x];
				if(c > 0){
					foreach(int b in body){
						if(b == c){
							tex.SetPixel(x,y,yellow);
						}
					}
				}
			}
		}
		tex.Apply();
		return tex;
	}

	public override void update (float deltaTime)
	{
		Map map = GameManager.Instance.map;
		if(mode == Modes.MOVE || mode == Modes.STOP){ //valid states for checking for input
			float nx = tile_x;
			float ny = tile_y;
			Dir nd = move_dir; //possible next input direction
			Dir ad = Dir.NONE; //possible aux direction (pre/post turning)

			//check each input direction in turn (with a bit of deadzone
			
			float deadzone = 0.05f;
			if( GameManager.Instance.horz < -deadzone || GameManager.Instance.inputLeft ){
				nx--;
				nd = Dir.LEFT;
				if(y < ny+0.5f){ //if off axis get correct direction to apply pre-post turning
					ad = Dir.DOWN;
				} else if(y > ny+0.5f){
					ad = Dir.UP;
				}
			} else if (GameManager.Instance.horz > deadzone || GameManager.Instance.inputRight){
				nx++;
				nd = Dir.RIGHT;
				if(y < ny+0.5f){
					ad = Dir.DOWN;
				} else if(y > ny+0.5f){
					ad = Dir.UP;
				}
			} else if (GameManager.Instance.vert > deadzone || GameManager.Instance.inputUp){
				ny--;
				nd = Dir.UP;
				if(x < nx+0.5f){
					ad = Dir.RIGHT;
				} else if(x > nx+0.5f){
					ad = Dir.LEFT;
				}
			} else if (GameManager.Instance.vert < -deadzone || GameManager.Instance.inputDown){
				ny++;
				nd = Dir.DOWN;
				if(x < nx+0.5f){
					ad = Dir.RIGHT;
				} else if(x > nx+0.5f){
					ad = Dir.LEFT;
				}
			}
			if(nd != move_dir){ //input direction is not the way we are going, check for turn
				int attempt_next_tile = map.getTileId(nx,ny);
				if(attempt_next_tile <= 0 || attempt_next_tile > 49) {
					//the tile in the input direction is not solid. Lets go!
					move_dir = nd; //new move direction is input direction
					updateDirection();
					next_tile.x = nx;
					next_tile.y = ny;
					aux_move_dir = ad; //if any pre-post turning needed, otherwise none
					animating = true;
					if(mode != Modes.MOVE) //get moving!
						setMode(Modes.MOVE);
				}
			}


			bool path_check = false;
			nx = tile_x;
			ny = tile_y;
			switch(move_dir){ //apply movement in the current move direction
				case Dir.RIGHT :
					x += deltaTime * maxSpeed * speed;
					if(x > next_tile.x+0.5f){ //we've hit the middle of the tile
						nx++; //we need this to check the next tile over, to see if it's solid
						path_check = true;
					}
					break;
				case Dir.DOWN :
					y += deltaTime * maxSpeed * speed;
					if(y > next_tile.y+0.5f){
						ny++;
						path_check = true;
					}
					break;
				case Dir.UP :
					y -= deltaTime * maxSpeed * speed;
					if(y < next_tile.y+0.5f){
						ny--;
						path_check = true;
					}
					break;
				case Dir.LEFT :
					x -= deltaTime * maxSpeed * speed;
					if(x < next_tile.x+0.5f){
						nx--;
						path_check = true;
					}
					break;
				default :
					//don't move
					break;
			}

			// aux movement for pre/post turning
			//pacman does not move in exactly the same manner as the ghosts.
			//ghosts always move to the centre of the tile before changing direction
			//due to user input, pacman can begin turning before getting to the centre.
			//this means pacman needs to apply some motion along the previous axis as well as the new axis. this is the auxiliary movement direction
			//this is called pre-turning (before we've hit the centre of the tile) or post-turning (after we've past the centre of the tile), in effect getting sucked into the turns
			//This allows pacman to gain some distance on the following ghosts by pre-turning (or losing ground by post-turning, you have to move back some distance already travelled)
			//it's only fractions of a single 8x8 pixel tile, but it adds up
			switch(aux_move_dir){
				case Dir.RIGHT :
					x += deltaTime * maxSpeed * speed;
					if(x >= next_tile.x+0.5f){
						x = next_tile.x+0.5f;
						aux_move_dir = Dir.NONE;
					}
					break;
				case Dir.DOWN :
					y += deltaTime * maxSpeed * speed;
					if(y >= next_tile.y+0.5f){
						y = next_tile.y+0.5f;
						aux_move_dir = Dir.NONE;
					}
					break;
				case Dir.UP :
					y -= deltaTime * maxSpeed * speed;
					if(y <= next_tile.y+0.5f){
						y = next_tile.y+0.5f;
						aux_move_dir = Dir.NONE;
					}
					break;
				case Dir.LEFT :
					x -= deltaTime * maxSpeed * speed;
					if(x <= next_tile.x+0.5f){
						x = next_tile.x+0.5f;
						aux_move_dir = Dir.NONE;
					}
					break;
				default :
					//don't move
					break;
			}

			if(path_check){
				int nt = map.getTileId(nx,ny);
				if(nt > 0 && nt < 50) { //the path directly in front is blocked. FULL STOP!
					setMode(Modes.STOP);
					next_tile.x = tile_x;
					next_tile.y = tile_y;
					x = tile_x + 0.5f;
					y = tile_y + 0.5f;
					move_dir = Dir.NONE;
					aux_move_dir = Dir.NONE;
				}
			}
		}

		base.update (deltaTime);

		if(last_x != tile_x || last_y != tile_y){ //is there something to be eaten here? Pacman eats everyting!
			map.eatTile(tile_x, tile_y);
		}
	}

	//adjust speed for FRIGHTENED mode
	public void ghostModeChanged(GhostMode _mode){
		if(_mode == GhostMode.FRIGHTENED){
			last_speed = speed;
			speed = fright_speeds[GameManager.Instance.levelIndex];
		} else {
			speed = last_speed;
		}
	}

	//check wether or not to naimate based on mode
	public void setMode(Modes m){
		mode = m;
		if(frames.ContainsKey(m)){
			anim_mode = mode;
			animating = true;
			anim_frame = 0;
			max_frames = frames[m].Length;
			if(m == Modes.DYING){
				move_dir = Dir.LEFT;
				updateDirection();
			}
		} else {
			animating = false;
		}
	}

	protected override void endOfAnim ()
	{
		if(anim_mode == Modes.DYING){
			animating = false; //only play death animation once
		}
		base.endOfAnim ();
	}

	protected override void updateDirection ()
	{
		//lazy direction change
		mesh.transform.eulerAngles = euler_rotations[move_dir];
		base.updateDirection ();
	}

	//switch the texture on the material... who are we kidding, it's the shader! Shaders rule!
	override protected void updateAnimFrame(){
		if(animating)
			if(frames.ContainsKey(anim_mode))
				if(anim_frame < frames[anim_mode].Length)
					mesh.renderer.material.SetTexture("_MainTex", frames[anim_mode][anim_frame]);
			
	}
}
